import tensorflow as tf
import numpy as np

# 기상을 예측해보자!

# Softmax 는 결과의 개수 만큼의 hypothesis 가 필요하다
# 현재 Rainy, Sunny, Cloudy 3개 임으로 3개에 해당하는 변수들을 선언해야한다.

# 1. Data를 data.csv 파일에서 불러오자

data = np.loadtxt('data.csv', delimiter=',', dtype=np.float32)

x1_data = data[:, 0]    # 습도 정보
x2_data = data[:, 1]    # 풍속 정보
x3_data = data[:, 2]    # 기온 정보

yR_data = data[:, 3]    # Rainy 이면 1 아니면 0
yS_data = data[:, 4]    # Sunny 이면 1 아니면 0
yC_data = data[:, 5]    # Cloudy 이면 1 아니면 0

# 2. 3개의 hypothesis 에 들어가는 변수들을 선언해 보자

# Rainy hypothesis 에 사용할 변수들
w1_R = tf.Variable(tf.random_normal([1]), name='rainyWeight1')     # Rainy hypothesis 에 사용할 습도에 관한 가중치
w2_R = tf.Variable(tf.random_normal([1]), name='rainyWeight2')     # Rainy hypothesis 에 사용할 가설에서풍속에 관한 가중치
w3_R = tf.Variable(tf.random_normal([1]), name='rainyWeight3')     # Rainy hypothesis 에 사용할 가설에서기온에 관한 가중치
b_R = tf.Variable(tf.random_normal([1]), name='rainyBias')         # Rainy hypothesis 에 사용할 편향 가중치

# Sunny hypothesis 에 사용할 변수들
w1_S = tf.Variable(tf.random_normal([1]), name='sunnyWeight1')     # Sunny hypothesis 에 사용할 습도에 관한 가중치
w2_S = tf.Variable(tf.random_normal([1]), name='sunnyWeight2')     # Sunny hypothesis 에 사용할 풍속에 관한 가중치
w3_S = tf.Variable(tf.random_normal([1]), name='sunnyWeight3')     # Sunny hypothesis 에 사용할 기온에 관한 가중치
b_S = tf.Variable(tf.random_normal([1]), name='sunnyBias')         # Sunny hypothesis 에 사용할 편향 가중치

# Cloudy hypothesis 에 사용할 변수들
w1_C = tf.Variable(tf.random_normal([1]), name='cloudyWeight1')    # Cloudy hypothesis 에 사용할 습도에 관한 가중치
w2_C = tf.Variable(tf.random_normal([1]), name='cloudyWeight2')    # Cloudy hypothesis 에 사용할 풍속에 관한 가중치
w3_C = tf.Variable(tf.random_normal([1]), name='cloudyWeight3')    # Cloudy hypothesis 에 사용할 기온에 관한 가중치
b_C = tf.Variable(tf.random_normal([1]), name='cloudyBias')        # Cloudy hypothesis 에 사용할 편향 가중치

# 3. 정규화
sess = tf.Session()
sess.run(tf.global_variables_initializer())

normed_x1_data = (x1_data - sess.run(tf.reduce_mean(x1_data))) / (max(x1_data) - min(x1_data))
normed_x2_data = (x2_data - sess.run(tf.reduce_mean(x2_data))) / (max(x2_data) - min(x2_data))
normed_x3_data = (x3_data - sess.run(tf.reduce_mean(x3_data))) / (max(x3_data) - min(x3_data))

# 4. 학습 알고리즘을 구성하자
hypothesis_R = tf.sigmoid(w1_R * normed_x1_data + w2_R * normed_x2_data + w3_R * normed_x3_data + b_R)
hypothesis_S = tf.sigmoid(w1_S * normed_x1_data + w2_S * normed_x2_data + w3_S * normed_x3_data + b_S)
hypothesis_C = tf.sigmoid(w1_C * normed_x1_data + w2_C * normed_x2_data + w3_C * normed_x3_data + b_C)

cost_R = -tf.reduce_mean(yR_data * tf.log(hypothesis_R) + (1 - yR_data) * tf.log(1 - hypothesis_R))
train_R = tf.train.GradientDescentOptimizer(learning_rate=0.01).minimize(cost_R)

cost_S = -tf.reduce_mean(yS_data * tf.log(hypothesis_S) + (1 - yS_data) * tf.log(1 - hypothesis_S))
train_S = tf.train.GradientDescentOptimizer(learning_rate=0.01).minimize(cost_S)

cost_C = -tf.reduce_mean(yC_data * tf.log(hypothesis_C) + (1 - yC_data) * tf.log(1 - hypothesis_C))
train_C = tf.train.GradientDescentOptimizer(learning_rate=0.01).minimize(cost_C)

# 5. 학습을 진행하자
for step in range(2001):
    sess.run(train_R)
    sess.run(train_S)
    sess.run(train_C)

# 6-1. 예측을 진행해보자
# 샘플정보
sample_x1_data = 1.0        # 습도 1.0
sample_x2_data = 2.3        # 풍속 2.3
sample_x3_data = 17         # 기온 17

# 6-2. 예측 데이터 정규화
normed_sample_x1_data = (sample_x1_data - sess.run(tf.reduce_mean(x1_data))) / (max(x1_data) - min(x1_data))
normed_sample_x2_data = (sample_x2_data - sess.run(tf.reduce_mean(x2_data))) / (max(x2_data) - min(x2_data))
normed_sample_x3_data = (sample_x3_data - sess.run(tf.reduce_mean(x3_data))) / (max(x3_data) - min(x3_data))

print("Rainy일 확률 : ", sess.run(tf.sigmoid(w1_R * normed_sample_x1_data + w2_R * normed_sample_x2_data + w3_R * normed_sample_x3_data + b_R)))
print("Sunny일 확률 : ", sess.run(tf.sigmoid(w1_S * normed_sample_x1_data + w2_S * normed_sample_x2_data + w3_S * normed_sample_x3_data + b_S)))
print("Cloudy일 확률 : ", sess.run(tf.sigmoid(w1_C * normed_sample_x1_data + w2_C * normed_sample_x2_data + w3_C * normed_sample_x3_data + b_C)))